"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HLSProxy = void 0;
const fastify_1 = __importDefault(require("fastify"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const m3u8_1 = __importDefault(require("@eyevinn/m3u8"));
;
class HLSProxy {
    constructor(handlers) {
        this.handlers = handlers;
        this.server = (0, fastify_1.default)({ ignoreTrailingSlash: true });
        this.server.get("/", () => __awaiter(this, void 0, void 0, function* () {
            return "OK\n";
        }));
        this.server.get("/*", (request, reply) => __awaiter(this, void 0, void 0, function* () {
            let basePath;
            const m = request.url.match("^(.*)/.*?$");
            if (m) {
                basePath = m[1] + "/";
            }
            const proxyRequest = {
                raw: request,
                basePath: basePath,
            };
            if (!this.handlers.originHandler) {
                throw new Error("No origin handler provided");
            }
            try {
                const originBaseUrl = yield this.handlers.originHandler(proxyRequest);
                if (request.url.match(/.m3u8/)) {
                    const m3u = yield this.fetchAndParseManifest(new URL(request.url, originBaseUrl));
                    let manifest = m3u.toString();
                    if (request.url.match(/\/master.m3u8/)) {
                        if (this.handlers.masterManifestHandler) {
                            manifest = yield this.handlers.masterManifestHandler(proxyRequest, new URL(originBaseUrl), m3u);
                        }
                    }
                    else {
                        if (this.handlers.mediaManifestHandler) {
                            manifest = yield this.handlers.mediaManifestHandler(proxyRequest, new URL(originBaseUrl), m3u);
                        }
                    }
                    reply.type("application/x-mpegURL").send(manifest);
                }
                else {
                    if (this.handlers.segmentRedirectHandler) {
                        const redirectDest = yield this.handlers.segmentRedirectHandler(proxyRequest, new URL(originBaseUrl));
                        reply.redirect(302, redirectDest);
                    }
                    else {
                        reply.code(404).send("Missing segment redirect handler");
                    }
                }
            }
            catch (err) {
                reply.code(500).send(err.message);
            }
        }));
    }
    fetchAndParseManifest(url) {
        return new Promise((resolve, reject) => {
            const parser = m3u8_1.default.createStream();
            parser.on("m3u", m3u => {
                resolve(m3u);
            });
            parser.on("error", err => {
                reject("Failed to parse master manifest: " + err);
            });
            (0, node_fetch_1.default)(url.href)
                .then(response => {
                if (response.status === 200) {
                    response.body.pipe(parser);
                }
                else {
                    reject("Failed to fetch master manifest");
                }
            });
        });
    }
    listen(port) {
        this.server.listen(port, "0.0.0.0", (err, address) => {
            if (err) {
                throw err;
            }
            console.log(`Server listening at ${address}`);
        });
    }
}
exports.HLSProxy = HLSProxy;
//# sourceMappingURL=index.js.map